/* This file is read_input.cc
   A member function of Lattice class which reads user input for MC simulation
   By Teck Leong Tan
*/

#include "lattice.h"
//#include "keywords.h"
//#include "filenames.h"

bool Lattice::read_input()
{
  /*The read_input() function reads user input from inputfile (see filenames.h)
    The various keywords are defined in filenames.h , of which 3 of the keywords
    are mandatory (NDIM, NTYPE, NBASIS) and their values have to be assigned in
    the inputfile before any other keywords.
  */
  
  /*The function does the following:
    1. Scans each line for a keyword (defined in keywords.h)
    2. Statements between 2 ! are treated as comments
    3. The relevant member data in the Lattice class are set correspondingly.
  */  
  
  ifstream input;
  input.open(inputfile, ios::in);//Opens input file
  if (!input)
  {
    cout<<"File: "<<inputfile<<" is not opened!"<<endl;
    exit(1);
  }  
  
  char word[256];
  int key_index;
  
  cout<<endl;
  while (1)
  { 
    input>>word;
    
    if (input.eof()) break;//Break on end of file
    if (input.fail())
    { 
      cerr<<"Unrecognised input format in "<<inputfile<<endl;
      cerr<<"Possible error in keyword: "<<key_array[key_index]<<endl;
      exit(1);
    }

    if (word[0]=='#')//Lines between two # are ignored
    {
      input.ignore(1024,'\n');
      continue;
    }

    else  
    { 
      //Loop over the no. of keywords
      for (int i=0;i<nkeys;i++)
      {
        //Look for keywords
        if (strcmp(word,key_array[i])==0)
        {
          key_index=i;
          break;
        }
        key_index=-1; //If no keyword is matched, key_index=-1;
      }
    
	    /* Sets the corresponding member data in Lattice class for 
         given keyword. Look up keywords.h for the correct keyword index.
      */
      if (key_index==0)//keyword NDIM
      {
        input>>ndim;
        input.ignore(256,'\n');
        //cout<<"NDIM= "<<ndim<<endl;
      }

      else if (key_index==1)//NTYPE
      {
        input>>num_type;
        if (num_type<2)
        {
          cout<<"Error in read_input.cc: No. of atom types has to be at least 2."<<endl;
	  exit(1);
        } 
        num_type--;
        input.ignore(256,'\n');
        //cout<<"NTYPE = "<<num_type<<endl;
      }

      else if (key_index==2)//NBASIS
      {
        input>>num_sublat;
        input.ignore(256,'\n');
        //cout <<"NBASIS= "<<num_sublat<<endl;      
      }

      else if (key_index==3)//RUNTYPE
      {
        char temp[30];
        input>>temp;
        canonical=temp[0]; 
        input.ignore(256,'\n');
        //cout <<"RUNTYPE ="<<temp<<" "<<canonical<<endl;
      }

      else if (key_index==4)//RUNSTART
      {
        input>>restart;
        // input.ignore(256,'\n');
        if (restart==2)
        {
          init_conc.assign(num_type,0);
          for (int i=0;i<num_type;i++)
          {
            input>>init_conc[i];
            //cout<<init_conc[i];
          } 
        }
        input.ignore(256,'\n');
        //cout<<"RUNSTART = "<<restart<<endl;
      }
      
      else if (key_index==5)//CONFIGFILE
      {
        cout<<"In key_index==5"<<endl;
        //delete inconf0;//Variable storing name of file of initial config
        //inconf0=new char[30];

        cout<<"In key_index==5"<<endl;
        char temp[30];

        input>>temp;
        strcpy (inconf0, temp);
        input.ignore(256,'\n');

        /*delete inconf0;//Variable storing name of file of initial config
        inconf0=new char;
        strcpy (inconf0, temp);*/ 

        cout<<"testing, inconf0= "<<inconf0<<endl;
      }

      else if (key_index==6)//CELLSIZE
      {
        //nrep is a vector storing no. of cells per direction
        nrep.assign(ndim, 0);//re-initialise nrep;
        nsize=1;//nsize is the total no. of atoms
        for (int i = 0; i<ndim;i++)
        {
          input>>nrep[i];
          nsize*=nrep[i];
          //cout<<"nrep["<<i<<"]= "<<nrep[i]<<endl;
          //cout<<"nrep size"<<nrep.size()<<endl;
        }
        input.ignore(256,'\n');

        /* factor[i] is used with nrep[i] to convert cartesian coordinates 
           to cell number (ijk)
        */
        factor.assign(ndim,1);   
        //Loop over no. off dimenstions (backwards)
        for (int i=ndim-1;i>=0;i-- )
        {
          for (int j=0;j<i;j++)
          {
            factor[i]*=nrep[j];
          }
        }//end for (int i=ndim-1;i>=0;i-- )
        //cout<<"CELLSIZE done"<<endl;
      }

      else if (key_index==7)//MAXBODY
      {
        input>>maxbody;//Max no. of interacting bodies
        input.ignore(256,'\n');
        //cout<<"MAXBODY done"<<endl;
      }

      else if (key_index==8)//RSEED
      {
        input>>randseed;//Seed for random no. generation
        input.ignore(256,'\n');
        //cout<<"RSEED = "<<randseed<<endl;
      }


      else if (key_index==9)//NPASS
      {
        input>>numpass;//num_pass is total no. of passes
        input.ignore(256,'\n');
        //cout<<"NPASS= "<<numpass<<endl ;
      }

      else if (key_index==10)//NEQUIL
      {
        input>>equil_pass;//No. of equilibration passes
        input.ignore(256,'\n');
        //cout<<"NEQUIL= "<<equil_pass<<endl;
      }

      else if (key_index==11)//TEMPERATURE
      {
        double T[3];
        for (int i=0; i<3;i++)
        {
          input>>T[i];
        }
        //input>>Tsub;//No. of subdivisions for interval
        input.ignore(256,'\n');
        Tinit=T[0];
        Tfinal=T[1];
        Tincrement=T[2];
        //Determine sign of Tincrement
        if(Tincrement<0){Tsign=-1;}
        else if(Tincrement>0){Tsign=1;}
        else {Tsign=0;}
        Tincrement=abs(Tincrement);//Tincrement term is hence positive
        //cout<<"TEMPERATURE: "<<Tinit<<", "<<Tfinal<<", "<<Tincrement<<endl;
      }

      else if (key_index==12) //CHEMPOT
      {
        mu_init.assign(num_type,0);
        mu_final.assign(num_type,0);
        mu_increment.assign(num_type,0);
        //cout<<"num_type= "<<num_type<<endl;        
        //input.ignore(256, '\n');
        for (int i=0;i<num_type;i++)
        {
          input>>mu_init[i]>>mu_final[i]>>mu_increment[i];
          input.ignore(256, '\n');
          //cout<<mu_init[i]<<" "<<mu_final[i]<<" "<<mu_increment[i]<<" "<<endl;
          //cout<<Lat.mu_init[i]<<" "<<Lat.mu_final[i]<<" "<<Lat.mu_increment[i]<<endl;
        }
        //cout<<key_array[key_index]<<" CHEMPOT done"<<endl;
      }

      else if (key_index==13)//SRO
      {
        char temp[10];       
        input>>temp;
        if (strcmp(temp,"cutoff")==0)//If option is cutoff
        {
          input>>pair_cutoff;
          pair_option=1;//Option for pair SRO
          correlation_option=1;//Turns on correlation mode
        }

        else if (strcmp(temp,"all")==0)//if option is all pairs
        {
          pair_option=2;
          correlation_option=1;//Turns on correlation mode
        }

        else
        {
          cout<<"Unrecognised input for keyword SROPAIR."<<endl;
          pair_option=0;
        }          
        input.ignore(256,'\n');
        //cout<<"SROPAIR done"<<endl;
      }

      else if (key_index==14)//LRO
      {
        char temp[3];
        input>>temp;

        if (strcmp(temp,"on")==0)//If LRO option is turned on
        {
          LRO_option=1;//Turns on LRO
          correlation_option=1;//Turns on correlation mode
        }
        else //If LRO option is turned off
        {
          LRO_option=0;
        }

        input.ignore(256, '\n');
        //cout<<"LRO done"<<endl;
      }//end else if (key_index==14)

      else if(key_index==15)//Phase diagram mode
      {
        char temp[3];
        input>>temp;

        if (strcmp(temp,"on")==0)//Phase diagram mode turned on
        {
            num_runs=2;//Do both cooling and heating runs per mu
        }
        else //If LRO option is turned off
        {
            num_runs=1;//Do only one run per mu
        }

        input.ignore(256, '\n');
      }//end else if (key_index==15)
     
      else if (key_index==16)//Structural inversion
      {
        char temp[3];
        input>>temp;

        if (strcmp(temp,"on")==0)//sinv_option is on
        {
            sinv_option=1;//Turns on sinv
        }
        else 
        {
          sinv_option=0;
        }

        input.ignore(256, '\n');
        //cout<<"LRO done"<<endl;
      }//end else if (key_index==16)

      else if (key_index==17)//Associate element to index
      {
        Element temp;
        input>>temp.element>>temp.index;
	elem_index.push_back(temp);
	input.ignore(256, '\n');
      }

      else if (key_index==18)//directory and filename to read lat. vectors
      {
        sinv_dir.clear();
        input>>sinv_dir;
	input.ignore(256, '\n');
      }	

      else if (key_index==19)//Tolerance used to determine if 2 positions are 
                             //equilvalent lattice sites
      {
        input>>fracnn2;
	fracnn2=fracnn2*fracnn2;//Fraction of the square of nearest neighbor used for tolerance
        input.ignore(256, '\n');
      }	

      else if (key_index==20)//Level (amount) of output
      {
        outlevel.clear();
        input>> outlevel;
        input.ignore(256, '\n');
      }	
      
      else if (key_index==21)//directory containing structure data
      {
        stdata_dir. clear();
        input>>stdata_dir;
	input.ignore(256, '\n');
      }	
      
      else //If does not match keyword
      {
        cout<<"Unrecognised keyword: "<<word<<endl;
        input.ignore(256, '\n');
      }
      //cout<<"default"<<endl;
    }
  }//end while not end of inputfile

  natoms=nsize * num_sublat;//Total no. of atoms=no. of cells * no. of sublat
  numpass+=equil_pass;//Total number of passes including equlibration
  cout<<"End of read_input()"<<endl;
}

//EOF
    
    
